using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections;
using System.Collections.Specialized;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;

namespace Superexpert {



    //*********************************************************************
    //
    // SuperDataGrid Class
    //
    // The SuperDataGrid control supports sorting, paging, editing, 
    // and deleting records through its
    // EnableSorting, EnablePaging, EnableEditing, and EnableDeleting 
    // properties.
    //
    // See the SuperDataGrid.aspx file for
    // samples of using this control.
    //
    // This control is from the book ASP.NET Unleashed written by 
    // Stephhen Walther and published by SAMS publishing.
    //
    //*********************************************************************

    public class SuperDataGrid : WebControl, IPostBackDataHandler, IPostBackEventHandler {

        private bool _enableSorting = true;
        private bool _enableEditing = true;
        private bool _enableDeleting = true;
        private bool _enablePaging = true;
        
        private DataTable _dataGridData = new DataTable();
        private DataView  _dataGridView = new DataView();
        private DataTable _dataGridSchema = new DataTable();
        private Hashtable colEditValues = new Hashtable();
        
        private bool _editClicked = false;
        private bool _updateClicked = false;
        private bool _deleteClicked = false;
        private int _deleteItemIndex = -1;

        private TableStyle _tableStyle = new TableStyle();
        private TableItemStyle _headerStyle = new TableItemStyle();
        private TableItemStyle _itemStyle = new TableItemStyle();
        private TableItemStyle _alternatingItemStyle = new TableItemStyle();
        private TableItemStyle _pagerStyle = new TableItemStyle();

        private string _commandText = String.Empty;
        private int _pageSize = 10;
        private string _prevText = "&lt;";
        private string _nextText = "&gt;";
        private string _pagerText = "Page {0} of {1}";

        private string _errorMessage = String.Empty;


        //*********************************************************************
        //
        // ConnectionString Property
        //
        // The Connection String for the database. Notice that this must
        // be a valid SQL connection string since the SuperDataGrid uses
        // the SqlClient classes.
        //
        //*********************************************************************

        public string ConnectionString {
            get { 
                if (ViewState["ConnectionString"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["ConnectionString"];
            }
            set { ViewState["ConnectionString"] = value; }
        }
    
    

        //*********************************************************************
        //
        // TableName Property
        //
        // The name of the database table to retrieve the data from. Instead
        // of using this property, you can use the CommandText property and
        // specify your own SELECT statement (with parameters).
        //
        //*********************************************************************

        public string TableName {
            get { 
                if (ViewState["TableName"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["TableName"];
            }
            set { ViewState["TableName"] = value; }
        }    
    
        

        //*********************************************************************
        //
        // CommandText Property
        //
        // The CommandText property enables you to specify a SELECT statement
        // to use when retrieving database data.
        //
        //*********************************************************************

        public string CommandText {
            get { 
                if (ViewState["CommandText"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["CommandText"];
            }
            set { ViewState["CommandText"] = value; }
        }    


        //*********************************************************************
        //
        // TableStyle Property
        //
        // The TableStyle object applied to the SuperDataGrid table. Use
        // this property to control the formatting of the main table.
        //
        //*********************************************************************

        public TableStyle TableStyle {
            get { return _tableStyle; }
        }



        //*********************************************************************
        //
        // HeaderStyle Property
        //
        // Use this property to control the formatting of the Header row.
        //
        //*********************************************************************

        public TableItemStyle HeaderStyle {
            get { return _headerStyle; }
        }



        //*********************************************************************
        //
        // ItemStyle Property
        //
        // Use this property to control the formatting of each row.
        //
        //*********************************************************************

        public TableItemStyle ItemStyle {
            get { return _itemStyle; }
        }
        


        //*********************************************************************
        //
        // AlternatingItemStyle Property
        //
        // Use this property to control the formatting of every other row.
        //
        //*********************************************************************

        public TableItemStyle AlternatingItemStyle {
            get { return _alternatingItemStyle; }
        }
                


        //*********************************************************************
        //
        // PagerStyle Property
        //
        // Use this property to control the formatting of the pager row.
        //
        //*********************************************************************

        public TableItemStyle PagerStyle {
            get { return _pagerStyle; }
        }


        //*********************************************************************
        //
        // SortColumn Property
        //
        // The column to sort on.
        //
        //*********************************************************************

        public string SortColumn {
            get { 
                if (ViewState["SortColumn"] == null)
                    return String.Empty;
                else
                    return (string)ViewState["SortColumn"];
            }
            set { ViewState["SortColumn"] = value; }
        }    



        //*********************************************************************
        //
        // SortOrder Property
        //
        // The Order to sort by (for example, ASC or DESC).
        //
        //*********************************************************************

        public string SortOrder {
            get { 
                if (ViewState["SortOrder"] == null)
                    return "ASC";
                else
                    return (string)ViewState["SortOrder"];
            }
            set { ViewState["SortOrder"] = value; }
        }    



        //*********************************************************************
        //
        // CurrentPageIndex Property
        //
        // The current page of records to display.
        //
        //*********************************************************************

        public int CurrentPageIndex {
            get { 
                if (ViewState["CurrentPageIndex"] == null)
                    return 1;
                else
                    return (int)ViewState["CurrentPageIndex"];
            }
            set { ViewState["CurrentPageIndex"] = value; }
        }    



        //*********************************************************************
        //
        // EnableSorting Property
        //
        // When true, columns can be sorted.
        //
        //*********************************************************************

        public bool EnableSorting {
            get { return _enableSorting; }
            set { _enableSorting = value; }
        }
        



        //*********************************************************************
        //
        // EnableEditing Property
        //
        // When true, rows can be edited.
        //
        //*********************************************************************

        public bool EnableEditing {
            get { return _enableEditing; }
            set { _enableEditing = value; }
        }
        

        //*********************************************************************
        //
        // EnableDeleting Property
        //
        // When true, rows can be deleted.
        //
        //*********************************************************************

        public bool EnableDeleting {
            get { return _enableDeleting; }
            set { _enableDeleting = value; }
        }




        //*********************************************************************
        //
        // PageSize Property
        //
        // The number of rows to display per page when paging is enabled.
        //
        //*********************************************************************

        public int PageSize {
            get { return _pageSize; }
            set { _pageSize = value; }
        }



        //*********************************************************************
        //
        // EditItemIndex Property
        //
        // The current row selected for editing.
        //
        //*********************************************************************

        public int EditItemIndex {
            get { 
                if (ViewState["EditItemIndex"] == null)
                    return -1;
                else
                    return (int)ViewState["EditItemIndex"];
            }
            set { ViewState["EditItemIndex"] = value; }
        }    



        //*********************************************************************
        //
        // LoadPostData Method
        //
        // When editing is enabled, and data is submitted, this method
        // updates the data stored in the colEditValues collection.
        //
        //*********************************************************************

        public bool LoadPostData(String postDataKey, NameValueCollection values) {
            foreach (string key in values.Keys)
                colEditValues[key] = values[key];
           return false;
        }


        //*********************************************************************
        //
        // RaisePostDataChangedEvent Method
        //
        // Required by the PostBackDataHandler interface.
        //
        //*********************************************************************

        public void RaisePostDataChangedEvent() {
        }



        //*********************************************************************
        //
        // RaisePostBackEvent Method
        //
        // When a link is clicked (for sorting, paging, editing), this
        // method launches off the right action.
        //
        //*********************************************************************

        public void RaisePostBackEvent(String eventArgument) {
            string[] parts = eventArgument.Split(new Char[] {'_'}, 2);

            switch (parts[0]) {
                case "edit" : 
                    _editClicked = true;
                    EditItemIndex = Int32.Parse(parts[1]);
                    break;
                case "cancel" :
                    EditItemIndex = -1;
                    break;
                case "update" :
                    _updateClicked = true;;
                    break;
                case "sort" :
                    UpdateSort( parts[1] );
                    break;
                case "page" :
                    UpdatePage( Int32.Parse(parts[1]) );
                    break;
                case "delete" :
                    _deleteClicked = true;
                    _deleteItemIndex = Int32.Parse(parts[1]);
                    break;
            }

        }


        //*********************************************************************
        //
        // UpdatePage Method
        //
        // Change the current page when paging is enabled.
        //
        //*********************************************************************
        
        private void UpdatePage(int newPageIndex) {
            CurrentPageIndex = newPageIndex;
        }
        


        //*********************************************************************
        //
        // UpdateDatabase Method
        //
        // Perform the update to the database table.
        //
        //*********************************************************************

        private void UpdateDatabase() {
            // Start by updating the data table
            DataRow row = _dataGridView[EditItemIndex].Row;
            foreach (DataColumn col in _dataGridData.Columns)
                if (colEditValues.Contains( "txt" + col.ColumnName)) {
                    try {                
                        row[col.ColumnName] = colEditValues["txt" + col.ColumnName];
                    } catch {
                        _errorMessage = "Could not update column " + col.ColumnName;
                        return;
                    }
                }

            // Create a command builder
            SqlDataAdapter dad = new SqlDataAdapter(_commandText, ConnectionString);
            SqlCommandBuilder builder = new SqlCommandBuilder(dad);
            try {
                dad.Update( _dataGridData);
            } catch (Exception ex){
                _errorMessage = ex.Message;
                return;
            }            
            
            // Deselect any row for editing
            EditItemIndex = -1;
        }


        private void DeleteRow(int rowIndex) {
            // Delete row from datatable
            DataRow row = _dataGridView[rowIndex].Row;
            row.Delete();

            // Create a command builder
            SqlDataAdapter dad = new SqlDataAdapter(_commandText, ConnectionString);
            SqlCommandBuilder builder = new SqlCommandBuilder(dad);
            try {
                dad.Update( _dataGridData);
            } catch (Exception ex){
                _errorMessage = ex.Message;
            }            
        }
        

        //*********************************************************************
        //
        // UpdateSort Method
        //
        // When a new column is clicked for sorting, update the SortColumn
        // and SortOrder.
        //
        //*********************************************************************

        private void UpdateSort(string sortExpression) {
            EditItemIndex = -1;
            CurrentPageIndex = 1;
            if (sortExpression == SortColumn) {
                if (SortOrder == "DESC")
                    SortOrder = "ASC";
                else
                    SortOrder = "DESC";
            } else
                SortOrder = "ASC";

            SortColumn = sortExpression;
        }



        //*********************************************************************
        //
        // InitDatabaseObjects Method
        //
        // Initialize the connection string and the command text.
        //
        //*********************************************************************

        private void InitDatabaseObjects() {
            // Check connection string
            if (ConnectionString == String.Empty)
                throw new ArgumentException( "You must set the ConnectionString property!");
        
            // Check table name
            if (TableName == String.Empty && CommandText == String.Empty)
                throw new ArgumentException( "You must set the TableName or CommandText property!");        

            // Calculate SELECT statement
            if (CommandText == String.Empty)
                _commandText = String.Format( "SELECT * FROM {0}", TableName );
            else
                _commandText = CommandText;
        }



        //*********************************************************************
        //
        // LoadDatabaseData Method
        //
        // Load the database data and schema.
        //
        //*********************************************************************

        private void LoadDatabaseData() {
            InitDatabaseObjects();

            // Get datagrid table
            SqlConnection con = new SqlConnection( ConnectionString );
            con.Open();
            SqlDataAdapter dad = new SqlDataAdapter( _commandText, con); 
            dad.Fill( _dataGridData );

                       
            // Get datagrid schema
            SqlCommand cmd = new SqlCommand(_commandText, con);
            _dataGridSchema = cmd.ExecuteReader(CommandBehavior.SchemaOnly).GetSchemaTable();
           
            con.Close();
        }
        



        //*********************************************************************
        //
        // RenderTable Method
        //
        // Display the main HTML table to display the database data.
        //
        //*********************************************************************

        private void RenderTable(HtmlTextWriter writer) {
                
            // Calculate Alternating Item Style
            _alternatingItemStyle.MergeWith(_itemStyle);


            
            // Calculate Start Index
            int startIndex = 0;
            if (_enablePaging)
                startIndex = ( (CurrentPageIndex - 1) * _pageSize);
                
            // Calculate End Index
            int endIndex = _dataGridData.Rows.Count;
            if (_enablePaging) {
                endIndex = startIndex + _pageSize;
                if (endIndex > _dataGridData.Rows.Count)
                    endIndex = _dataGridData.Rows.Count;
            }    


            // Render rows        
            for (int i=startIndex;i < endIndex; i++) {
                if (i % 2 == 0)
                    _itemStyle.AddAttributesToRender(writer);
                else
                    _alternatingItemStyle.AddAttributesToRender(writer);


                if (_enableEditing && EditItemIndex == i)
                    RenderEditRow(writer, i);
                else
                    RenderRow(writer, i);
            }
        }



        //*********************************************************************
        //
        // RenderRow Method
        //
        // Display a particular row in the HTML table.
        //
        //*********************************************************************

        private void RenderRow(HtmlTextWriter writer, int itemIndex) {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
                if (_enableEditing) {
                    writer.RenderBeginTag(HtmlTextWriterTag.Td);
                    RenderLink(writer, "Edit", "edit_" + itemIndex.ToString());
                    
                    if (_enableDeleting) {
                        writer.Write( "&nbsp;|&nbsp;");
                        writer.AddAttribute(HtmlTextWriterAttribute.Onclick, "return confirm('Are you sure you want to delete this row?')");
                        
                        RenderLink(writer, "Delete", "delete_" + itemIndex.ToString());
                    }
                    
                    writer.RenderEndTag();
                }
            
                for (int i=0;i < _dataGridData.Columns.Count; i++ ) {
                    writer.RenderBeginTag( HtmlTextWriterTag.Td);
                    RenderColumn(writer, itemIndex, i);
                    writer.RenderEndTag();
                }
            writer.RenderEndTag();
        }
        


        //*********************************************************************
        //
        // RenderEditRow Method
        //
        // Render a row for editing.
        //
        //*********************************************************************

        private void RenderEditRow(HtmlTextWriter writer, int itemIndex) {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
                writer.RenderBeginTag(HtmlTextWriterTag.Td);

                // Render update link
                RenderLink(writer, "Update", "update_" + itemIndex.ToString());

                writer.Write( "&nbsp;|&nbsp;");

                // Render Cancel link
                RenderLink(writer, "Cancel", "cancel_" + itemIndex.ToString());

                writer.RenderEndTag();
            
                for (int i=0;i < _dataGridData.Columns.Count; i++ ) {
                    writer.AddAttribute(HtmlTextWriterAttribute.Valign, "top");
                    writer.RenderBeginTag( HtmlTextWriterTag.Td);
                    RenderEditColumn(writer, itemIndex, i);
                    writer.RenderEndTag();
                }
            writer.RenderEndTag();
        }



        //*********************************************************************
        //
        // RenderColumn Method
        //
        // Render the value of a column. For money columns, we'll apply 
        // some formatting.
        //
        //*********************************************************************

        private void RenderColumn(HtmlTextWriter writer, int rowIndex, int colIndex) {
            object colValue = _dataGridView[rowIndex][colIndex];
            SqlDbType providerType = ((SqlDbType)_dataGridSchema.Rows[colIndex]["ProviderType"]);

            switch (providerType) {
                case SqlDbType.Money :
                    writer.Write( ((decimal)colValue).ToString("c") );
                    break;
                case SqlDbType.Image :
                    writer.Write( "[Image]" );
                    break;
                case SqlDbType.DateTime :
                    DateTime dtm = (DateTime)colValue;
                    if ( dtm.Date == dtm )
                        writer.Write( dtm.ToString("d"));
                    else
                        writer.Write( dtm.ToString("g"));
                    break;
                default:
                    writer.Write( colValue.ToString() );
                    break;
            }        
        }



        //*********************************************************************
        //
        // RenderEditColumn Method
        //
        // Render a column with the right UI for editing its contents.
        //
        //*********************************************************************

        private void RenderEditColumn(HtmlTextWriter writer, int rowIndex, int colIndex) {
            string colName = _dataGridData.Columns[colIndex].ColumnName;
            object colValue = _dataGridView[rowIndex][colIndex];
            if (colEditValues.Contains(colName) )
                colValue = colEditValues[colName];
            SqlDbType providerType = ((SqlDbType)_dataGridSchema.Rows[colIndex]["ProviderType"]);
            int columnSize = ((int)_dataGridSchema.Rows[colIndex]["ColumnSize"]); 
            bool colIsIdentity = ((bool)_dataGridSchema.Rows[colIndex]["IsIdentity"]);

            if (colIsIdentity)
                writer.Write( colValue.ToString() );            
            else 
                switch (providerType) {
                    case SqlDbType.VarChar :
                    case SqlDbType.NVarChar :
                    case SqlDbType.NText :
                    case SqlDbType.Text :
                        if (columnSize > 100 )
                            RenderTextBox(writer, colName, colValue, 30, 3, columnSize);
                        else if (columnSize > 30)
                            RenderTextBox(writer, colName, colValue, 30, 1, columnSize);
                        else
                            RenderTextBox(writer, colName, colValue, columnSize, 1, columnSize);
                        break;
                    case SqlDbType.Bit :
                        RenderSelect(writer, colName);
                        break;
                    case SqlDbType.Image :
                        writer.Write( "[Image]" );
                        break;
                    case SqlDbType.DateTime :
                        DateTime dtm = (DateTime)colValue;
                        if ( dtm.Date == dtm )
                            RenderTextBox(writer, colName, dtm.ToString("d"), 10, 1, 20);
                        else
                            RenderTextBox(writer, colName, dtm.ToString("g"), 20, 1, 30);
                        break;
                    default:
                        RenderTextBox(writer, colName, colValue, 5, 1, columnSize);
                        break;
                }        
        }



        //*********************************************************************
        //
        // RenderTextBox Method
        //
        // Display a textbox.
        //
        //*********************************************************************

        private void RenderTextBox(HtmlTextWriter writer, string colName, object colValue, int columns, int rows, int maxLength) {
            string name = "txt" + colName;
            if (rows > 1) {
                writer.AddAttribute(HtmlTextWriterAttribute.Id, name);
                writer.AddAttribute(HtmlTextWriterAttribute.Name, name);
                writer.AddAttribute(HtmlTextWriterAttribute.Cols, columns.ToString());
                writer.AddAttribute(HtmlTextWriterAttribute.Rows, rows.ToString());
                writer.RenderBeginTag(HtmlTextWriterTag.Textarea);
                writer.Write(colValue);
                writer.RenderEndTag();
            } else
                writer.Write( String.Format( "<input type=\"text\" name=\"{0}\" id=\"{0}\" value=\"{1}\" size=\"{2}\" maxlength=\"{3}\">", name, colValue, columns, maxLength) );
        }


        //*********************************************************************
        //
        // RenderSelect Method
        //
        // Display an HTML select list.
        //
        //*********************************************************************

        private void RenderSelect(HtmlTextWriter writer, string colName) {
            string name = "txt" + colName;
            bool selected = false;           
            if (colEditValues[name].ToString() == "True")
                selected = true;
                
            writer.AddAttribute(HtmlTextWriterAttribute.Id, name);
            writer.AddAttribute(HtmlTextWriterAttribute.Name, name);
            writer.RenderBeginTag(HtmlTextWriterTag.Select);
            writer.AddAttribute(HtmlTextWriterAttribute.Value, "True");
            if (selected)
                writer.AddAttribute(HtmlTextWriterAttribute.Selected, "SELECTED");
            writer.RenderBeginTag(HtmlTextWriterTag.Option);
            writer.Write("True");
            writer.RenderEndTag();
            writer.AddAttribute(HtmlTextWriterAttribute.Value, "False");
            if (!selected)
                writer.AddAttribute(HtmlTextWriterAttribute.Selected, "SELECTED");
            writer.RenderBeginTag(HtmlTextWriterTag.Option);
            writer.Write("False");
            writer.RenderEndTag();
            writer.RenderEndTag();
        }


        //*********************************************************************
        //
        // RenderLink Method
        //
        // Display a link that causes a postback.
        //
        //*********************************************************************

        private void RenderLink(HtmlTextWriter writer, string label, string argument) {
            writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:" + Page.GetPostBackEventReference(this, argument ) );
            writer.RenderBeginTag(HtmlTextWriterTag.A);
            writer.Write(label);
            writer.RenderEndTag();
        }


        //*********************************************************************
        //
        // RenderHeaderRow Method
        //
        // Display the first row in the HTML table. This row either displays
        // just the column names or column names that can be clicked and sorted.
        //
        //*********************************************************************

        private void RenderHeaderRow(HtmlTextWriter writer) {
            HeaderStyle.AddAttributesToRender(writer);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            if (_enableEditing) {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.Write("&nbsp;");
                writer.RenderEndTag();
            }

            if (_enableSorting)
                RenderSortHeaderRow(writer);
            else
                RenderNormalHeaderRow(writer);            
            writer.RenderEndTag();
        }


        //*********************************************************************
        //
        // RenderNormalHeaderRow Method
        //
        // Display all the column names.
        //
        //*********************************************************************

        private void RenderNormalHeaderRow(HtmlTextWriter writer) {
            foreach (DataColumn col in _dataGridData.Columns) {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                writer.Write( col.ColumnName );
                writer.RenderEndTag();   
            }
        }

        //*********************************************************************
        //
        // RenderSortHeaderRow Method
        //
        // Display all the column names as links for sorting columns.
        //
        //*********************************************************************

        private void RenderSortHeaderRow(HtmlTextWriter writer) {
            Style _sortStyle = new Style();
            _sortStyle.CopyFrom(HeaderStyle);
            foreach (DataColumn col in _dataGridData.Columns) {
                writer.RenderBeginTag(HtmlTextWriterTag.Td);
                _sortStyle.AddAttributesToRender(writer);
                RenderLink(writer, col.ColumnName, "sort_" + col.ColumnName);
                writer.RenderEndTag();   
            }
        }


        //*********************************************************************
        //
        // RenderPager Method
        //
        // Display the pager user interface.
        //
        //*********************************************************************

        private void RenderPager(HtmlTextWriter writer) {
            // Calculate total records
            int totalRecords = _dataGridData.Rows.Count;

            // if less rows than PageSize then exit
            if (_pageSize > totalRecords)
                return;         

            // Calculate page count
            int pageCount = totalRecords / _pageSize;
        

            // Now do a mod for any remainder
            if ((totalRecords % _pageSize) > 0)
                pageCount ++;

       

            // calculate column count
            int columnCount = _dataGridData.Columns.Count;
            if (_enableEditing)
                columnCount ++;
        
            // Render Pager row
            _pagerStyle.AddAttributesToRender(writer);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.AddAttribute(HtmlTextWriterAttribute.Colspan, columnCount.ToString() );
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            
            // Render pager table
            writer.AddAttribute(HtmlTextWriterAttribute.Width, "100%");
            writer.AddAttribute(HtmlTextWriterAttribute.Cellpadding, "3");
            writer.AddAttribute(HtmlTextWriterAttribute.Cellspacing, "0");
            writer.RenderBeginTag(HtmlTextWriterTag.Table);
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            
            // render previous link
            if (CurrentPageIndex > 1)
                RenderLink(writer, _prevText, "page_" + (CurrentPageIndex -1) );
            else
                writer.Write( _prevText);
            writer.Write("&nbsp;");

            // Render each page number
            for (int pageIndex=1;pageIndex <= pageCount;pageIndex++) {
                writer.Write( "&nbsp;" );
                if (pageIndex == CurrentPageIndex) {
                    writer.RenderBeginTag(HtmlTextWriterTag.B);
                    writer.Write(pageIndex.ToString());
                    writer.RenderEndTag();
                } else {
                    RenderLink(writer, pageIndex.ToString(), "page_" + pageIndex.ToString() );
                }
                writer.Write( "&nbsp;" );
            }


            // render next link
            writer.Write("&nbsp;");
            if (CurrentPageIndex < pageCount)
                RenderLink(writer, _nextText, "page_" + (CurrentPageIndex + 1).ToString() );
            else
                writer.Write( _nextText);

            
            // Show pager text
            writer.AddAttribute(HtmlTextWriterAttribute.Align, "right");
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.Write(String.Format(_pagerText, CurrentPageIndex, pageCount));
            writer.RenderEndTag();
            
            writer.RenderEndTag(); // end cell
            writer.RenderEndTag(); // end row
            writer.RenderEndTag(); // end table
            
            
            writer.RenderEndTag(); // end pager row
            writer.RenderEndTag(); // end pager table
        }


        //*********************************************************************
        //
        // RenderError Method
        //
        // Display an error message in a JavaScript alert box.
        //
        //*********************************************************************

        private void RenderError(HtmlTextWriter writer) {
            writer.AddAttribute("language", "javascript");
            writer.RenderBeginTag(HtmlTextWriterTag.Script);
            writer.Write( String.Format(" alert(\"{0}\"); ", _errorMessage ));
            writer.RenderEndTag();
        }


        //*********************************************************************
        //
        // OnPreRender Method
        //
        // Enable smart navigation (so the Grid will scroll back to the
        // right place on postback) and register the control to require
        // postback event notifications.
        //
        //*********************************************************************

        protected override void OnPreRender(EventArgs e) {
            Page.SmartNavigation = true;
            Page.RegisterRequiresPostBack(this);

            // Get the Data
            LoadDatabaseData();

            // Sort the data table
            _dataGridView = _dataGridData.DefaultView;
            if (_enableSorting && SortColumn != String.Empty)
                _dataGridView.Sort = String.Format("{0} {1}", SortColumn, SortOrder);
            
            
            // Get the values for editing
            if (_editClicked) {
                for (int i=0;i < _dataGridData.Columns.Count;i++)
                    colEditValues["txt" + _dataGridData.Columns[i].ColumnName] = _dataGridView[EditItemIndex][i];
            }

            // Update the database with any edits
            if (_updateClicked) 
                UpdateDatabase();
                
            // Update the database with any deletes
            if (_deleteClicked)
                DeleteRow(_deleteItemIndex);    
        }


        //*********************************************************************
        //
        // RenderContents Method
        //
        // The main method that kicks off the rendering of the SuperDataGrid.
        //
        //*********************************************************************
        
        protected override void RenderContents(HtmlTextWriter writer) {


            // Display any error message
            if (_errorMessage != String.Empty)
                RenderError(writer);

            // Render opening table tag
            _tableStyle.AddAttributesToRender(writer);
            writer.RenderBeginTag(HtmlTextWriterTag.Table);
            
            // Render Header Row
            RenderHeaderRow( writer );
            
            // Render the HTML table
            RenderTable( writer);
            
            // Render Pager
            if (_enablePaging)
                RenderPager( writer );

            // Render End Table Tag
            writer.RenderEndTag();
        }




        //*********************************************************************
        //
        // SuperDataGrid Constructor
        //
        // Set defaults for formatting (user can override).
        //
        //*********************************************************************

        public SuperDataGrid() {
            TableStyle.CellSpacing = 0;
            TableStyle.CellPadding = 4;
            TableStyle.GridLines = GridLines.Vertical;
            TableStyle.BorderColor = ColorTranslator.FromHtml("#999999");
            TableStyle.BorderStyle = BorderStyle.Solid;
            TableStyle.BorderWidth = new Unit("1px");
            
            HeaderStyle.BackColor = ColorTranslator.FromHtml("#000084");
            HeaderStyle.ForeColor = Color.White;
            HeaderStyle.Font.Bold = true;
            
            ItemStyle.BackColor = ColorTranslator.FromHtml("#EEEEEE");
            AlternatingItemStyle.BackColor = Color.Gainsboro;
            
            PagerStyle.BackColor = ColorTranslator.FromHtml("#999999");
        }    
    
    
    }


}
